import json
import os
from django.contrib.auth.models import User
from django.db.models import Q
from django.urls import reverse
from rest_framework import serializers
import jsonschema
from uhepp_vault.models import PUBLIC_LEVEL, INTERNAL_LEVEL, PUBLIC_LEVEL, \
                               VISIBILITY_LEVELS, Plot, Collection
from .masks import filter_collections

class UserSerializer(serializers.HyperlinkedModelSerializer):
    collections = serializers.HyperlinkedRelatedField(
        'collection-detail',
        many=True,
        read_only=True
    )
    visibility = serializers.ChoiceField(
        source="profile.visibility",
        choices=VISIBILITY_LEVELS,
    )

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', "collections",
                  "visibility"]

class UserListSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        'user-detail',
        lookup_field='username'
    )
    class Meta:
        model = User
        fields = ['username', 'url']

class CollectionListSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(read_only=True)
    owner = serializers.HyperlinkedRelatedField(
        'user-detail',
        lookup_field='username',
        read_only=True,
    )
    class Meta:
        model = Collection
        fields = ["id", "title", "owner", "url"]

class CollectionSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(read_only=True)
    owner = serializers.HyperlinkedRelatedField(
        'user-detail',
        lookup_field='username',
        read_only=True,
    )
    plots = serializers.HyperlinkedRelatedField(
        'plot-detail',
        many=True,
        read_only=True
    )
    class Meta:
        model = Collection
        fields = ["id", "title", "owner", "url", "visibility", "description", "plots"]

class PlotListSerializer(serializers.HyperlinkedModelSerializer):
    uuid = serializers.UUIDField(read_only=True)
    class Meta:
        model = Plot
        fields = ["uuid", "collection", "url"]

class PlotSerializer(serializers.HyperlinkedModelSerializer):
    uuid = serializers.UUIDField(read_only=True)
    collection = serializers.HyperlinkedRelatedField(
        'collection-detail',
        queryset=Collection.objects
    )

    def validate_uhepp(self, value):
        """Check if the JSON data conforms to the uhepp schema"""
        try:
            schema_path = os.path.join(
                os.path.dirname(__file__),
                "uhepp.schema.json"
            )

            with open(schema_path) as schema_file:
                schema = json.load(schema_file)

            jsonschema.validate(value, schema=schema)
        except Exception as e:
            raise serializers.ValidationError("JSON schema validation error: %s" % e)

        return value

    class Meta:
        model = Plot
        fields = ["uuid", "collection", "url", "uhepp"]
